////////////////////////////////////////////////////////////////////////
// Arduino Bluetooth Interface with Mindset
// 
// This is example code provided by NeuroSky and is provided
// license free.
////////////////////////////////////////////////////////////////////////

#include <SoftwareSerial.h>       

// Output pin definitions
#define rxPin 2                          
#define txPin 3
#define pwm5  5

SoftwareSerial Serial2 =  SoftwareSerial(rxPin, txPin);      // set up a software serial port, blocked reads

// checksum variables
byte generatedChecksum = 0;
byte readChecksum = 0;

// payload values
int payloadLength = 0;
byte payloadData[10] = {0};
byte poor_quality = 0;
byte attention = 0;
byte meditation = 0;

// system variables
byte initial_onhead = 0;

//////////////////////////
// Microprocessor Setup //
//////////////////////////
void setup()
{ 
  pinMode(rxPin,INPUT);
  pinMode(txPin,OUTPUT);
  analogWrite(pwm5,0);      
  
  Serial.begin(9600);       // USB serial
  Serial2.begin(9600);      // Bluetooth serial
  
  Serial2.print(33, BYTE);    // send command 0x29 to MindSet to enable poor out
}

////////////////////////////////
// Read data from Serial UART //
////////////////////////////////
byte ReadOneByte(){
  int ByteRead;
  
  ByteRead = Serial2.read();     // read from the Bluetooth Serial
  Serial.print((char)ByteRead);  // eacho the same byte out the USB serial (for debug purposes)
  
  return ByteRead;
}

/////////////
//MAIN LOOP//
/////////////
void loop()
{
  // reset the variables
  generatedChecksum = 0;        
  poor_quality = 0;
  attention = 0;
  meditation = 0;
      
  // Packet Parsing
  if(ReadOneByte() == 170){    // if it is 0xAA then check the next byte for 0xAA
    if(ReadOneByte() == 170){   // if there is a second 0xAA byte then we know we have a real packet
    
      payloadLength = ReadOneByte();  // read the payload length
      
      // The software UART requires quick reading of all bytes at once
      for(int i = 0; i < (payloadLength + 1); i++){  
        payloadData[i] = ReadOneByte();    // read one byte into temporary storage
      }
      
      // All reading should be completed by now, do some processing on the data
      for(int i = 0; i < payloadLength; i++){  
        generatedChecksum += payloadData[i];
      }

      // Take one's compliment of generated checksum
      generatedChecksum = 255 - generatedChecksum;
      
      readChecksum = payloadData[payloadLength];

      if(readChecksum == generatedChecksum){    // if the checksum matches, packet is valid
      
          // Parse the data - all data is received correctly            
          for(int i = 0; i < payloadLength; i++){  
          // Sample input data: AA AA 06 ==> 02 C8 04 00 05 00 CS
        
            switch (payloadData[i]) {
              case 2:
                i++;
                poor_quality = payloadData[i]; // poor_quality (200 for poorest quality, 0 for usable quality)
                break;
              case 4:
                i++;
                attention = payloadData[i];    // range is 0-100 (100 for most attention)
                break;
              case 5:
                i++;
                meditation = payloadData[i];   // range is 0-100 (100 for most meditation)
                break;
              default:
                Serial.print('I');
            }
          }  // end for loop reading payload data
            
          // Data received successfully, control system with eSense values  
          
          // Insert User Code here.
          
          // Initial on head detection will make motor move a little bit
          if(poor_quality == 0 && initial_onhead == 0){    // poor quality reached 0, system functional
            analogWrite(pwm5,255);      
            delay(100);
            analogWrite(pwm5,0);    
            initial_onhead = 1;  
          }else if(initial_onhead == 1 && poor_quality < 200){
            // Do this as long as poor_quality is less than 200
            // Atention value above a threshold will make motor move in proportion to Attention level
            if(attention < 20){
              analogWrite(pwm5,0);    
            }else{
              analogWrite(pwm5,(int)attention*2.55);  // calculated range is 0 to 260, it can accept 255
            }
          }else if(poor_quality == 200){
             // it is offhead i.e. poor_quality reaches 200, reset state
             initial_onhead = 0;
             analogWrite(pwm5,0); 
          }
      }else{
        // Checksum Error
        Serial.print('E');
        Serial.print('E');
      }  // end if else for checksum
      
    } // end if read 0xAA byte
  } // end if read 0xAA byte

  // test line for setting the motor to turn on without receiving packets
  // analogWrite(pwm5,40); // test line for motorf
}
